@@ -1,6 +1,7 @@
 /*
  *  Copyright (C) 2004, 2005, 2006, 2009, 2008, 2009, 2010 Savoir-Faire Linux Inc.
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
+ *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -36,35 +37,41 @@ int framesPerBufferAlsa = 2048;
 
 // Constructor
 AlsaLayer::AlsaLayer (ManagerImpl* manager)
-        : AudioLayer (manager , ALSA)
-        , _PlaybackHandle (NULL)
-        , _CaptureHandle (NULL)
-        , _periodSize()
-        , _audioPlugin()
-        , IDSoundCards()
-        , _is_prepared_playback (false)
-        , _is_prepared_capture (false)
-        , _is_running_playback (false)
-        , _is_running_capture (false)
-        , _is_open_playback (false)
-        , _is_open_capture (false)
-        , _trigger_request (false)
-        , _audioThread (NULL)
+    : AudioLayer (manager , ALSA)
+    , _PlaybackHandle (NULL)
+    , _RingtoneHandle (NULL)
+    , _CaptureHandle (NULL)
+    , _periodSize (160)
+    , _audioPlugin ("default")
+    , IDSoundCards()
+    , _is_prepared_playback (false)
+    , _is_prepared_capture (false)
+    , _is_running_playback (false)
+    , _is_running_capture (false)
+    , _is_open_playback (false)
+    , _is_open_capture (false)
+    , _trigger_request (false)
+    , _audioThread (NULL)
 
 {
-    _debug (" Constructor of AlsaLayer called");
+    _debug ("Audio: Build ALSA layer");
     /* Instanciate the audio thread */
     // _audioThread = new AudioThread (this);
     // _audioThread = NULL;
     _urgentRingBuffer.createReadPointer();
 
-    dcblocker = new DcBlocker();
+    _audioPlugin = AudioLayer::_manager->audioPreference.getPlugin();
+
+    AudioLayer::_noisesuppressstate = true;
+
+    // captureFile = new ofstream ("probeFile", ofstream::binary);
+
 }
 
 // Destructor
 AlsaLayer::~AlsaLayer (void)
 {
-    _debug ("Destructor of AlsaLayer called");
+    _debug ("Audio: Destroy of ALSA layer");
     closeLayer();
 
     if (_converter) {
@@ -72,43 +79,41 @@ AlsaLayer::~AlsaLayer (void)
         _converter = NULL;
     }
 
-    if (dcblocker) {
-        delete dcblocker;
-        dcblocker = NULL;
-    }
+    // captureFile->close();
+    // delete captureFile;
 }
 
 bool
 AlsaLayer::closeLayer()
 {
-    _debugAlsa ("Close ALSA streams");
+    _debugAlsa ("Audio: Close ALSA streams");
 
     try {
         /* Stop the audio thread first */
         if (_audioThread) {
-            _debug ("Stop Audio Thread");
+            _debug ("Audio: Stop Audio Thread");
             delete _audioThread;
             _audioThread=NULL;
         }
     } catch (...) {
-        _debugException ("! ARTP Exception: when stopping audiortp");
+        _debugException ("Audio: Exception: when stopping audiortp");
         throw;
     }
 
+
     /* Then close the audio devices */
     closeCaptureStream();
-
     closePlaybackStream();
 
-    _CaptureHandle = 0;
-
-    _PlaybackHandle = 0;
+    _CaptureHandle = NULL;
+    _PlaybackHandle = NULL;
+    _RingtoneHandle = NULL;
 
     return true;
 }
 
 bool
-AlsaLayer::openDevice (int indexIn, int indexOut, int sampleRate, int frameSize, int stream , std::string plugin)
+AlsaLayer::openDevice (int indexIn, int indexOut, int indexRing, int sampleRate, int frameSize, int stream , std::string plugin)
 {
     /* Close the devices before open it */
     if (stream == SFL_PCM_BOTH && is_capture_open() == true && is_playback_open() == true) {
@@ -119,59 +124,72 @@ AlsaLayer::openDevice (int indexIn, int indexOut, int sampleRate, int frameSize,
     else if ( (stream == SFL_PCM_PLAYBACK || stream == SFL_PCM_BOTH) && is_playback_open () == true)
         closePlaybackStream ();
 
-
     _indexIn = indexIn;
-
     _indexOut = indexOut;
+    _indexRing = indexRing;
 
     _audioSampleRate = sampleRate;
-
     _frameSize = frameSize;
 
     _audioPlugin = std::string (plugin);
 
-    _debugAlsa (" Setting AlsaLayer: device     in=%2d, out=%2d", _indexIn, _indexOut);
-
+    _debugAlsa (" Setting AlsaLayer: device     in=%2d, out=%2d, ring=%2d", _indexIn, _indexOut, _indexRing);
     _debugAlsa ("                   : alsa plugin=%s", _audioPlugin.c_str());
-
     _debugAlsa ("                   : nb channel in=%2d, out=%2d", _inChannel, _outChannel);
-
     _debugAlsa ("                   : sample rate=%5d, format=%s", _audioSampleRate, SFLDataFormatString);
 
     _audioThread = NULL;
 
-    ost::MutexLock lock (_mutex);
-
-    std::string pcmp = buildDeviceTopo (plugin, indexOut, 0);
-
-    std::string pcmc = buildDeviceTopo (plugin, indexIn, 0);
-
     // use 1 sec buffer for resampling
     _converter = new SamplerateConverter (_audioSampleRate, 1000);
 
-    // open_device (pcmp, pcmc, stream);
-    return true; // open_device (pcmp, pcmc, stream);
+    AudioLayer::_dcblocker = new DcBlocker();
+    AudioLayer::_audiofilter = new AudioProcessing (static_cast<Algorithm *> (_dcblocker));
+
+
+    return true;
 }
 
 void
 AlsaLayer::startStream (void)
 {
-    _debug ("AlsaLayer:: startStream");
+    _debug ("Audio: Start stream");
 
-    std::string pcmp = buildDeviceTopo (_audioPlugin, _indexOut, 0);
-    std::string pcmc = buildDeviceTopo (_audioPlugin, _indexIn, 0);
+    if (_audiofilter)
+        _audiofilter->resetAlgorithm();
+
+    if (is_playback_running() && is_capture_running())
+        return;
+
+    std::string pcmp;
+    std::string pcmr;
+    std::string pcmc;
+
+    if (_audioPlugin == PCM_DMIX_DSNOOP) {
+        pcmp = buildDeviceTopo (PCM_DMIX, _indexOut, 0);
+        pcmr = buildDeviceTopo (PCM_DMIX, _indexRing, 0);
+        pcmc = buildDeviceTopo (PCM_DSNOOP, _indexIn, 0);
+    } else {
+        pcmp = buildDeviceTopo (_audioPlugin, _indexOut, 0);
+        pcmr = buildDeviceTopo (_audioPlugin, _indexRing, 0);
+        pcmc = buildDeviceTopo (_audioPlugin, _indexIn, 0);
+    }
+
+    _debug ("pcmp: %s, index %d", pcmp.c_str(), _indexOut);
+    _debug ("pcmr: %s, index %d", pcmr.c_str(), _indexRing);
+    _debug ("pcmc: %s, index %d", pcmc.c_str(), _indexIn);
 
     if (!is_playback_open()) {
-        open_device (pcmp, pcmc, SFL_PCM_PLAYBACK);
+        open_device (pcmp, pcmc, pcmr, SFL_PCM_PLAYBACK);
     }
 
     if (!is_capture_open()) {
-        open_device (pcmp, pcmc, SFL_PCM_CAPTURE);
+        open_device (pcmp, pcmc, pcmr, SFL_PCM_CAPTURE);
     }
 
     prepareCaptureStream ();
-
     preparePlaybackStream ();
+
     startCaptureStream ();
     startPlaybackStream ();
 
@@ -183,7 +201,7 @@ AlsaLayer::startStream (void)
 
     if (_audioThread == NULL) {
         try {
-            _debug ("Start Audio Thread");
+            _debug ("Audio: Start Audio Thread");
             _audioThread = new AudioThread (this);
             _audioThread->start();
         } catch (...) {
@@ -196,24 +214,27 @@ AlsaLayer::startStream (void)
 void
 AlsaLayer::stopStream (void)
 {
-    _debug ("AlsaLayer:: stopStream");
+    _debug ("Audio: Stop stream");
 
     try {
         /* Stop the audio thread first */
         if (_audioThread) {
-            _debug ("Stop Audio Thread");
+            _debug ("Audio: Stop audio thread");
             delete _audioThread;
             _audioThread=NULL;
         }
     } catch (...) {
-        _debugException ("! ARTP Exception: when stopping audiortp");
+        _debugException ("Audio: Exception: when stopping audiortp");
         throw;
     }
 
     closeCaptureStream ();
-
     closePlaybackStream ();
 
+    _PlaybackHandle = NULL;
+    _CaptureHandle = NULL;
+    _RingtoneHandle = NULL;
+
     /* Flush the ring buffers */
     flushUrgent ();
     flushMain ();
@@ -231,6 +252,14 @@ bool AlsaLayer::isCaptureActive (void)
         return false;
 }
 
+void AlsaLayer::setNoiseSuppressState (bool state)
+{
+    // if a stream already opened
+    AudioLayer::_noisesuppressstate = state;
+
+}
+
+
 //////////////////////////////////////////////////////////////////////////////////////////////
 /////////////////   ALSA PRIVATE FUNCTIONS   ////////////////////////////////////////////////
 ////////////////////////////////////////////////////////////////////////////////////////////
@@ -240,10 +269,10 @@ void AlsaLayer::stopCaptureStream (void)
     int err;
 
     if (_CaptureHandle) {
-        _debug ("AlsaLayer:: stop Alsa capture");
+        _debug ("Audio: Stop ALSA capture");
 
         if ( (err = snd_pcm_drop (_CaptureHandle)) < 0)
-            _debug ("AlsaLayer:: Error stopping ALSA capture: %s", snd_strerror (err));
+            _debug ("Audio: Error: stopping ALSA capture: %s", snd_strerror (err));
         else
             stop_capture ();
 
@@ -258,10 +287,10 @@ void AlsaLayer::closeCaptureStream (void)
         stopCaptureStream ();
 
     if (is_capture_open()) {
-        _debug ("AlsaLayer:: close ALSA capture");
+        _debug ("Audio: Close ALSA capture");
 
         if ( (err = snd_pcm_close (_CaptureHandle)) < 0)
-            _debug ("Error closing ALSA capture: %s", snd_strerror (err));
+            _debug ("Audio: Error: Closing ALSA capture: %s", snd_strerror (err));
         else
             close_capture ();
     }
@@ -272,7 +301,7 @@ void AlsaLayer::startCaptureStream (void)
     int err;
 
     if (_CaptureHandle && !is_capture_running()) {
-        _debug ("AlsaLayer:: start ALSA capture");
+        _debug ("Audio: Start ALSA capture");
 
         if ( (err = snd_pcm_start (_CaptureHandle)) < 0)
             _debug ("Error starting ALSA capture: %s",  snd_strerror (err));
@@ -286,10 +315,10 @@ void AlsaLayer::prepareCaptureStream (void)
     int err;
 
     if (is_capture_open() && !is_capture_prepared()) {
-        _debug ("AlsaLayer:: prepare ALSA capture");
+        _debug ("Audio: Prepare ALSA capture");
 
         if ( (err = snd_pcm_prepare (_CaptureHandle)) < 0)
-            _debug ("Error preparing ALSA capture: %s", snd_strerror (err));
+            _debug ("Audio: Error: preparing ALSA capture: %s", snd_strerror (err));
         else
             prepare_capture ();
     }
@@ -299,11 +328,19 @@ void AlsaLayer::stopPlaybackStream (void)
 {
     int err;
 
+    if (_RingtoneHandle && is_playback_running()) {
+        _debug ("Audio: Stop ALSA ringtone");
+
+        if ( (err = snd_pcm_drop (_RingtoneHandle)) < 0) {
+            _debug ("Audio: Error: Stop ALSA ringtone: %s", snd_strerror (err));
+        }
+    }
+
     if (_PlaybackHandle && is_playback_running()) {
-        _debug ("AlsaLayer:: stop ALSA playback");
+        _debug ("Audio: Stop ALSA playback");
 
         if ( (err = snd_pcm_drop (_PlaybackHandle)) < 0)
-            _debug ("Error stopping ALSA playback: %s", snd_strerror (err));
+            _debug ("Audio: Error: Stopping ALSA playback: %s", snd_strerror (err));
         else
             stop_playback ();
     }
@@ -317,14 +354,23 @@ void AlsaLayer::closePlaybackStream (void)
     if (is_playback_prepared() == true && is_playback_running() == true)
         stopPlaybackStream ();
 
+
     if (is_playback_open()) {
-        _debug ("AlsaLayer:: close ALSA playback");
+
+        _debug ("Audio: Close ALSA playback");
+
+        if (_RingtoneHandle) {
+            if ( (err = snd_pcm_close (_RingtoneHandle)) < 0) {
+                _warn ("Audio: Error: Closing ALSA ringtone: %s", snd_strerror (err));
+            }
+        }
 
         if ( (err = snd_pcm_close (_PlaybackHandle)) < 0)
-            _debug ("Error closing ALSA playback: %s", snd_strerror (err));
+            _warn ("Audio: Error: Closing ALSA playback: %s", snd_strerror (err));
         else
             close_playback ();
     }
+
 }
 
 void AlsaLayer::startPlaybackStream (void)
@@ -332,10 +378,10 @@ void AlsaLayer::startPlaybackStream (void)
     int err;
 
     if (_PlaybackHandle && !is_playback_running()) {
-        _debug ("AlsaLayer:: start ALSA playback");
+        _debug ("Audio: Start ALSA playback");
 
         if ( (err = snd_pcm_start (_PlaybackHandle)) < 0)
-            _debug ("Error starting ALSA playback: %s", snd_strerror (err));
+            _debug ("Audio: Error: Starting ALSA playback: %s", snd_strerror (err));
         else
             start_playback();
     }
@@ -346,10 +392,10 @@ void AlsaLayer::preparePlaybackStream (void)
     int err;
 
     if (is_playback_open() && !is_playback_prepared()) {
-        _debug ("AlsaLayer:: prepare playback stream");
+        _debug ("Audio: Prepare playback stream");
 
         if ( (err = snd_pcm_prepare (_PlaybackHandle)) < 0)
-            _debug ("Error preparing the device: %s", snd_strerror (err));
+            _debug ("Audio: Preparing the device: %s", snd_strerror (err));
         else
             prepare_playback ();
     }
@@ -395,21 +441,22 @@ bool AlsaLayer::alsa_set_params (snd_pcm_t *pcm_handle, int type, int rate)
     int err;
     int format;
     int periods = 4;
-    int periodsize = 2048;
+    int periodsize = 160;
 
     /* Allocate the snd_pcm_hw_params_t struct */
     snd_pcm_hw_params_malloc (&hwparams);
 
-    _periodSize = 2048;
+    // _periodSize = periodsize;
+    _periodSize = periodsize;
     /* Full configuration space */
 
     if ( (err = snd_pcm_hw_params_any (pcm_handle, hwparams)) < 0) {
-        _debugAlsa (" Cannot initialize hardware parameter structure (%s)", snd_strerror (err));
+        _debugAlsa ("Audio: Error: Cannot initialize hardware parameter structure (%s)", snd_strerror (err));
         return false;
     }
 
     if ( (err = snd_pcm_hw_params_set_access (pcm_handle, hwparams, SND_PCM_ACCESS_RW_INTERLEAVED)) < 0) {
-        _debugAlsa (" Cannot set access type (%s)", snd_strerror (err));
+        _debugAlsa ("Audio: Error: Cannot set access type (%s)", snd_strerror (err));
         return false;
     }
 
@@ -417,7 +464,7 @@ bool AlsaLayer::alsa_set_params (snd_pcm_t *pcm_handle, int type, int rate)
     format = SND_PCM_FORMAT_S16_LE;
 
     if ( (err = snd_pcm_hw_params_set_format (pcm_handle, hwparams, (snd_pcm_format_t) format)) < 0) {
-        _debugAlsa (" Cannot set sample format (%s)", snd_strerror (err));
+        _debugAlsa ("Audio: Error: Cannot set sample format (%s)", snd_strerror (err));
         return false;
     }
 
@@ -429,17 +476,18 @@ bool AlsaLayer::alsa_set_params (snd_pcm_t *pcm_handle, int type, int rate)
     exact_ivalue = rate;
 
     if ( (err = snd_pcm_hw_params_set_rate_near (pcm_handle, hwparams, &exact_ivalue, &dir) < 0)) {
-        _debugAlsa (" Cannot set sample rate (%s)", snd_strerror (err));
+        _debugAlsa ("Audio: Error: Cannot set sample rate (%s)", snd_strerror (err));
         return false;
-    }
+    } else
+        _debug ("Audio: Set audio rate to %d", rate);
 
     if (dir!= 0) {
-        _debugAlsa (" (%i) The choosen rate %d Hz is not supported by your hardware.Using %d Hz instead. ",type ,rate, exact_ivalue);
+        _debugAlsa ("Audio: Error: (%i) The choosen rate %d Hz is not supported by your hardware.Using %d Hz instead. ",type ,rate, exact_ivalue);
     }
 
     /* Set the number of channels */
     if ( (err = snd_pcm_hw_params_set_channels (pcm_handle, hwparams, 1)) < 0) {
-        _debugAlsa (" Cannot set channel count (%s)", snd_strerror (err));
+        _debugAlsa ("Audio: Error: Cannot set channel count (%s)", snd_strerror (err));
         return false;
     }
 
@@ -449,12 +497,12 @@ bool AlsaLayer::alsa_set_params (snd_pcm_t *pcm_handle, int type, int rate)
     dir=0;
 
     if ( (err = snd_pcm_hw_params_set_period_size_near (pcm_handle, hwparams, &exact_lvalue , &dir)) < 0) {
-        _debugAlsa (" Cannot set period time (%s)", snd_strerror (err));
+        _debugAlsa ("Audio: Error: Cannot set period time (%s)", snd_strerror (err));
         return false;
     }
 
     if (dir!=0) {
-        _debugAlsa ("(%i) The choosen period size %d bytes is not supported by your hardware.Using %d instead. ", type, (int) periodsize, (int) exact_lvalue);
+        _debugAlsa ("Audio: Warning: (%i) The choosen period size %d bytes is not supported by your hardware.Using %d instead. ", type, (int) periodsize, (int) exact_lvalue);
     }
 
     periodsize = exact_lvalue;
@@ -465,12 +513,12 @@ bool AlsaLayer::alsa_set_params (snd_pcm_t *pcm_handle, int type, int rate)
     dir=0;
 
     if ( (err = snd_pcm_hw_params_set_periods_near (pcm_handle, hwparams, &exact_ivalue, &dir)) < 0) {
-        _debugAlsa (" Cannot set periods number (%s)", snd_strerror (err));
+        _debugAlsa ("Audio: Error: Cannot set periods number (%s)", snd_strerror (err));
         return false;
     }
 
     if (dir!=0) {
-        _debugAlsa (" The choosen period number %i bytes is not supported by your hardware.Using %i instead. ", periods, exact_ivalue);
+        _debugAlsa ("Audio: Warning: The choosen period number %i bytes is not supported by your hardware.Using %i instead. ", periods, exact_ivalue);
     }
 
     periods=exact_ivalue;
@@ -478,7 +526,7 @@ bool AlsaLayer::alsa_set_params (snd_pcm_t *pcm_handle, int type, int rate)
     /* Set the hw parameters */
 
     if ( (err = snd_pcm_hw_params (pcm_handle, hwparams)) < 0) {
-        _debugAlsa (" Cannot set hw parameters (%s)", snd_strerror (err));
+        _debugAlsa ("Audio: Error: Cannot set hw parameters (%s)", snd_strerror (err));
         return false;
     }
 
@@ -490,13 +538,13 @@ bool AlsaLayer::alsa_set_params (snd_pcm_t *pcm_handle, int type, int rate)
 
     /* Set the start threshold */
 
-    if ( (err = snd_pcm_sw_params_set_start_threshold (pcm_handle, swparams, 2700 /*periodsize*2*/)) < 0) {
-        _debugAlsa (" Cannot set start threshold (%s)", snd_strerror (err));
+    if ( (err = snd_pcm_sw_params_set_start_threshold (pcm_handle, swparams, _periodSize*2)) < 0) {
+        _debugAlsa ("Audio: Error: Cannot set start threshold (%s)", snd_strerror (err));
         return false;
     }
 
     if ( (err = snd_pcm_sw_params (pcm_handle, swparams)) < 0) {
-        _debugAlsa (" Cannot set sw parameters (%s)", snd_strerror (err));
+        _debugAlsa ("Audio: Error: Cannot set sw parameters (%s)", snd_strerror (err));
         return false;
     }
 
@@ -508,46 +556,60 @@ bool AlsaLayer::alsa_set_params (snd_pcm_t *pcm_handle, int type, int rate)
 
 
 bool
-AlsaLayer::open_device (std::string pcm_p, std::string pcm_c, int flag)
+AlsaLayer::open_device (std::string pcm_p, std::string pcm_c, std::string pcm_r, int flag)
 {
 
     int err;
 
     if (flag == SFL_PCM_BOTH || flag == SFL_PCM_PLAYBACK) {
 
-        _debug ("AlsaLayer:: open playback device");
-        // if((err = snd_pcm_open(&_PlaybackHandle, pcm_p.c_str(),  SND_PCM_STREAM_PLAYBACK, 0 )) < 0){
+        _debug ("Audio: Open playback device (and ringtone)");
 
-        if ( (err = snd_pcm_open (&_PlaybackHandle, pcm_p.c_str(),  SND_PCM_STREAM_PLAYBACK, 0)) < 0) {
-            _debugAlsa ("Error while opening playback device %s",  pcm_p.c_str());
+        if ( (err = snd_pcm_open (&_PlaybackHandle, pcm_p.c_str(), SND_PCM_STREAM_PLAYBACK, 0)) < 0) {
+            _warn ("Audio: Error while opening playback device %s",  pcm_p.c_str());
             setErrorMessage (ALSA_PLAYBACK_DEVICE);
             close_playback ();
             return false;
         }
 
         if (!alsa_set_params (_PlaybackHandle, 1, getSampleRate())) {
-            _debug ("playback failed");
+            _warn ("Audio: Error: Playback failed");
             snd_pcm_close (_PlaybackHandle);
             close_playback ();
             return false;
         }
 
+        if (getIndexOut() != getIndexRing()) {
+
+            if ( (err = snd_pcm_open (&_RingtoneHandle, pcm_r.c_str(), SND_PCM_STREAM_PLAYBACK, 0)) < 0) {
+                _warn ("Audio: Error: Opening ringtone device %s", pcm_r.c_str());
+                // setErrorMessage(ALSA_RINGTONE_DEVICE);
+            }
+
+            if (!alsa_set_params (_RingtoneHandle, 1, getSampleRate())) {
+                _warn ("Audio: Error: Ringtone failed");
+                snd_pcm_close (_RingtoneHandle);
+
+            }
+        }
+
         open_playback ();
     }
 
     if (flag == SFL_PCM_BOTH || flag == SFL_PCM_CAPTURE) {
 
-        _debug ("AlsaLayer:: open capture device");
+        _debug ("Audio: Open capture device");
 
         if ( (err = snd_pcm_open (&_CaptureHandle,  pcm_c.c_str(),  SND_PCM_STREAM_CAPTURE, 0)) < 0) {
-            _debugAlsa ("Error while opening capture device %s",  pcm_c.c_str());
+            _warn ("Audio: Error: Opening capture device %s",  pcm_c.c_str());
+
             setErrorMessage (ALSA_CAPTURE_DEVICE);
             close_capture ();
             return false;
         }
 
         if (!alsa_set_params (_CaptureHandle, 0, 8000)) {
-            _debug ("capture failed");
+            _warn ("Audio: Error: Capture failed");
             snd_pcm_close (_CaptureHandle);
             close_capture ();
             return false;
@@ -558,32 +620,25 @@ AlsaLayer::open_device (std::string pcm_p, std::string pcm_c, int flag)
         // prepare_capture ();
     }
 
-    /* Start the secondary audio thread for callbacks */
-    /*
-    try {
-        _audioThread->start();
-    } catch (...) {
-        _debugException ("Fail to start audio thread");
-    }
-    */
+
 
     return true;
 }
 
 //TODO	first frame causes broken pipe (underrun) because not enough data are send --> make the handle wait to be ready
 int
-AlsaLayer::write (void* buffer, int length)
+AlsaLayer::write (void* buffer, int length, snd_pcm_t * handle)
 {
     if (_trigger_request == true) {
         _trigger_request = false;
         startPlaybackStream ();
     }
 
-    snd_pcm_uframes_t frames = snd_pcm_bytes_to_frames (_PlaybackHandle, length);
+    snd_pcm_uframes_t frames = snd_pcm_bytes_to_frames (handle, length);
 
     int err;
 
-    if ( (err=snd_pcm_writei (_PlaybackHandle , buffer , frames)) <0) {
+    if ( (err=snd_pcm_writei (handle, buffer , frames)) <0) {
         switch (err) {
 
             case -EPIPE:
@@ -592,17 +647,17 @@ AlsaLayer::write (void* buffer, int length)
 
             case -EIO:
                 //_debugAlsa(" XRUN playback ignored (%s)", snd_strerror(err));
-                handle_xrun_playback();
+                handle_xrun_playback (handle);
 
-                if (snd_pcm_writei (_PlaybackHandle , buffer , frames) <0)
-                    _debugAlsa ("XRUN handling failed");
+                if (snd_pcm_writei (handle, buffer , frames) <0)
+                    _debugAlsa ("Audio: XRUN handling failed");
 
                 _trigger_request = true;
 
                 break;
 
             default:
-                _debugAlsa ("Write error unknown - dropping frames **********************************: %s", snd_strerror (err));
+                _debugAlsa ("Audio: Write error unknown - dropping frames: %s", snd_strerror (err));
 
                 stopPlaybackStream ();
 
@@ -635,14 +690,14 @@ AlsaLayer::read (void* buffer, int toCopy)
             case -ESTRPIPE:
 
             case -EIO:
-                _debugAlsa (" XRUN capture ignored (%s)", snd_strerror (samples));
+                _debugAlsa ("Audio: XRUN capture ignored (%s)", snd_strerror (samples));
                 handle_xrun_capture();
                 //samples = snd_pcm_readi( _CaptureHandle, buffer, frames);
                 //if (samples<0)  samples=0;
                 break;
 
             case EPERM:
-                _debugAlsa (" Capture EPERM (%s)", snd_strerror (samples));
+                _debugAlsa ("Audio: Capture EPERM (%s)", snd_strerror (samples));
                 prepareCaptureStream ();
                 startCaptureStream ();
                 break;
@@ -662,7 +717,7 @@ AlsaLayer::read (void* buffer, int toCopy)
 void
 AlsaLayer::handle_xrun_capture (void)
 {
-    _debugAlsa ("handle_xrun_capture");
+    _debugAlsa ("Audio: Handle xrun capture");
 
     snd_pcm_status_t* status;
     snd_pcm_status_alloca (&status);
@@ -676,23 +731,25 @@ AlsaLayer::handle_xrun_capture (void)
             startCaptureStream ();
         }
     } else
-        _debugAlsa (" Get status failed");
+        _debugAlsa ("Audio: Get status failed");
 }
 
 void
-AlsaLayer::handle_xrun_playback (void)
+AlsaLayer::handle_xrun_playback (snd_pcm_t *handle)
 {
-    _debugAlsa ("AlsaLayer:: handle_xrun_playback");
+    _debugAlsa ("Audio: Handle xrun playback");
 
     int state;
     snd_pcm_status_t* status;
     snd_pcm_status_alloca (&status);
 
-    if ( (state = snd_pcm_status (_PlaybackHandle, status)) < 0)   _debugAlsa (" Error: Cannot get playback handle status (%s)" , snd_strerror (state));
+    if ( (state = snd_pcm_status (handle, status)) < 0)
+        _debugAlsa ("Audio: Error: Cannot get playback handle status (%s)" , snd_strerror (state));
     else {
         state = snd_pcm_status_get_state (status);
 
         if (state  == SND_PCM_STATE_XRUN) {
+            _debug ("Audio: audio device in state SND_PCM_STATE_XRUN, restart device");
             stopPlaybackStream ();
             preparePlaybackStream ();
 
@@ -705,10 +762,10 @@ AlsaLayer::handle_xrun_playback (void)
 std::string
 AlsaLayer::buildDeviceTopo (std::string plugin, int card, int subdevice)
 {
-    std::string pcm = plugin;
     std::stringstream ss,ss1;
+    std::string pcm = plugin;
 
-    if (pcm == "default" || pcm == "pulse")
+    if (pcm == PCM_DEFAULT)
         return pcm;
 
     ss << card;
@@ -723,6 +780,8 @@ AlsaLayer::buildDeviceTopo (std::string plugin, int card, int subdevice)
         pcm.append (ss1.str());
     }
 
+    _debug ("Audio: Device topo: %s", pcm.c_str());
+
     return pcm;
 }
 
@@ -732,6 +791,8 @@ AlsaLayer::getSoundCardsInfo (int stream)
     std::vector<std::string> cards_id;
     HwIDPair p;
 
+    _debug ("Audio: Get sound cards info: ");
+
     snd_ctl_t* handle;
     snd_ctl_card_info_t *info;
     snd_pcm_info_t* pcminfo;
@@ -768,6 +829,7 @@ AlsaLayer::getSoundCardsInfo (int stream)
                     // The number of the sound card is associated with a string description
                     p = HwIDPair (numCard , description);
                     IDSoundCards.push_back (p);
+
                 }
             }
 
@@ -824,67 +886,80 @@ AlsaLayer::soundCardGetIndex (std::string description)
 void AlsaLayer::audioCallback (void)
 {
 
-    int toGet, urgentAvailBytes, normalAvailBytes, maxBytes;
+    int toGet, urgentAvailBytes, normalAvailBytes;
     unsigned short spkrVolume, micVolume;
     AudioLoop *tone;
     AudioLoop *file_tone;
 
-    SFLDataFormat *out;
-    SFLDataFormat *rsmpl_out;
+    SFLDataFormat *out = NULL;
+    SFLDataFormat *rsmpl_out = NULL;
 
     spkrVolume = _manager->getSpkrVolume();
     micVolume  = _manager->getMicVolume();
 
-    /*
-    int writeableSize = snd_pcm_avail_update(_PlaybackHandle);
-    _debug("writeableSize %i", writeableSize);
-    */
+    tone = _manager->getTelephoneTone();
+    file_tone = _manager->getTelephoneFile();
 
     // AvailForGet tell the number of chars inside the buffer
     // framePerBuffer are the number of data for one channel (left)
     urgentAvailBytes = _urgentRingBuffer.AvailForGet();
 
+    if (!_PlaybackHandle || !_CaptureHandle)
+        return;
+
+    snd_pcm_wait (_PlaybackHandle, 20);
+
+    int playbackAvailSmpl = snd_pcm_avail_update (_PlaybackHandle);
+    int playbackAvailBytes = playbackAvailSmpl*sizeof (SFLDataFormat);
+    // _debug("PLAYBACK: %d", playbackAvailSmpl);
+
     if (urgentAvailBytes > 0) {
 
         // Urgent data (dtmf, incoming call signal) come first.
-        toGet = (urgentAvailBytes < (int) (framesPerBufferAlsa * sizeof (SFLDataFormat))) ? urgentAvailBytes : framesPerBufferAlsa * sizeof (SFLDataFormat);
-        out = (SFLDataFormat*) malloc (toGet * sizeof (SFLDataFormat));
-        _urgentRingBuffer.Get (out, toGet, spkrVolume);
+        toGet = (urgentAvailBytes < (int) (playbackAvailBytes)) ? urgentAvailBytes : playbackAvailBytes;
+        out = (SFLDataFormat*) malloc (toGet);
+        memset (out, 0, toGet);
 
-        /* Play the sound */
-        write (out, toGet);
+        if (out) {
+            _urgentRingBuffer.Get (out, toGet, spkrVolume);
+            write (out, toGet, _PlaybackHandle);
+            free (out);
+        }
 
-        free (out);
-        out=0;
+        out=NULL;
 
         // Consume the regular one as well (same amount of bytes)
         getMainBuffer()->discard (toGet);
 
     } else {
 
-        tone = _manager->getTelephoneTone();
-        file_tone = _manager->getTelephoneFile();
+        normalAvailBytes = getMainBuffer()->availForGet();
 
-        toGet = framesPerBufferAlsa;
-        maxBytes = toGet * sizeof (SFLDataFormat);
+        if (tone && (normalAvailBytes <= 0)) {
 
-        if (tone != 0) {
+            out = (SFLDataFormat *) malloc (playbackAvailBytes);
+            memset (out, 0, playbackAvailBytes);
 
-            out = (SFLDataFormat*) malloc (maxBytes * sizeof (SFLDataFormat));
-            tone->getNext (out, toGet, spkrVolume);
-            write (out , maxBytes);
+            if (out) {
+                tone->getNext (out, playbackAvailSmpl, spkrVolume);
+                write (out , playbackAvailBytes, _PlaybackHandle);
+                free (out);
+            }
 
-            free (out);
-            out = 0;
+            out = NULL;
 
-        } else if (file_tone != 0) {
+        } else if (file_tone && !_RingtoneHandle && (normalAvailBytes <= 0)) {
 
-            out = (SFLDataFormat*) malloc (maxBytes * sizeof (SFLDataFormat));
-            file_tone->getNext (out, toGet, spkrVolume);
-            write (out , maxBytes);
+            out = (SFLDataFormat *) malloc (playbackAvailBytes);
+            memset (out, 0, playbackAvailBytes);
 
-            free (out);
-            out = 0;
+            if (out) {
+                file_tone->getNext (out, playbackAvailSmpl, spkrVolume);
+                write (out, playbackAvailBytes, _PlaybackHandle);
+                free (out);
+            }
+
+            out = NULL;
 
         } else {
 
@@ -892,30 +967,23 @@ void AlsaLayer::audioCallback (void)
             // If nothing urgent, play the regular sound samples
 
             int _mainBufferSampleRate = getMainBuffer()->getInternalSamplingRate();
-            int maxNbSamplesToGet = 0;
-            int maxNbBytesToGet = 0;
+            int maxNbSamplesToGet = playbackAvailSmpl;
+            int maxNbBytesToGet = playbackAvailBytes;
 
             // Compute maximal value to get into the ring buffer
 
             if (_mainBufferSampleRate && ( (int) _audioSampleRate != _mainBufferSampleRate)) {
 
                 double upsampleFactor = (double) _audioSampleRate / _mainBufferSampleRate;
-
-                maxNbSamplesToGet = (int) ( (double) framesPerBufferAlsa / upsampleFactor);
-
-
-            } else {
-
-                maxNbSamplesToGet = framesPerBufferAlsa;
+                maxNbSamplesToGet = (int) ( (double) playbackAvailSmpl  / upsampleFactor);
+                maxNbBytesToGet = maxNbSamplesToGet * sizeof (SFLDataFormat);
 
             }
 
-            maxNbBytesToGet = maxNbSamplesToGet * sizeof (SFLDataFormat);
-
-            normalAvailBytes = getMainBuffer()->availForGet();
             toGet = (normalAvailBytes < (int) maxNbBytesToGet) ? normalAvailBytes : maxNbBytesToGet;
 
             out = (SFLDataFormat*) malloc (maxNbBytesToGet);
+            memset (out, 0, maxNbBytesToGet);
 
             if (normalAvailBytes) {
 
@@ -923,66 +991,95 @@ void AlsaLayer::audioCallback (void)
 
                 if (_mainBufferSampleRate && ( (int) _audioSampleRate != _mainBufferSampleRate)) {
 
-
-                    rsmpl_out = (SFLDataFormat*) malloc (framesPerBufferAlsa * sizeof (SFLDataFormat));
-
                     // Do sample rate conversion
                     int nb_sample_down = toGet / sizeof (SFLDataFormat);
 
-                    int nbSample = _converter->upsampleData ( (SFLDataFormat*) out, rsmpl_out, _mainBufferSampleRate, _audioSampleRate, nb_sample_down);
+                    if (rsmpl_out) {
+                        rsmpl_out = (SFLDataFormat*) malloc (playbackAvailBytes);
+                        memset (out, 0, playbackAvailBytes);
 
+                        int nbSample = _converter->upsampleData ( (SFLDataFormat*) out, rsmpl_out, _mainBufferSampleRate, _audioSampleRate, nb_sample_down);
+                        write (rsmpl_out, nbSample*sizeof (SFLDataFormat), _PlaybackHandle);
+                        free (rsmpl_out);
+                    }
 
-
-                    write (rsmpl_out, nbSample*sizeof (SFLDataFormat));
-
-                    free (rsmpl_out);
                     rsmpl_out = 0;
 
                 } else {
 
-                    write (out, toGet);
+                    write (out, toGet, _PlaybackHandle);
 
                 }
 
             } else {
 
-                if ( (tone == 0) && (file_tone == 0)) {
+                if (!tone && !file_tone) {
 
-                    SFLDataFormat* zeros = (SFLDataFormat*) malloc (framesPerBufferAlsa * sizeof (SFLDataFormat));
+                    SFLDataFormat *zeros = (SFLDataFormat*) malloc (playbackAvailBytes);
 
-                    bzero (zeros, framesPerBufferAlsa * sizeof (SFLDataFormat));
-                    write (zeros, framesPerBufferAlsa * sizeof (SFLDataFormat));
+                    if (zeros) {
+                        bzero (zeros, playbackAvailBytes);
+                        write (zeros, playbackAvailBytes, _PlaybackHandle);
+                        free (zeros);
+                    }
 
-                    free (zeros);
+                    zeros = NULL;
                 }
             }
 
             _urgentRingBuffer.Discard (toGet);
 
             free (out);
-            out = 0;
+            out = NULL;
 
         }
 
     }
 
+    if (file_tone && _RingtoneHandle) {
+
+        int ringtoneAvailSmpl = snd_pcm_avail_update (_RingtoneHandle);
+        int ringtoneAvailBytes = ringtoneAvailSmpl*sizeof (SFLDataFormat);
+
+        // _debug("RINGTONE: %d", ringtoneAvailSmpl);
+
+        out = (SFLDataFormat *) malloc (ringtoneAvailBytes);
+
+        if (out) {
+            file_tone->getNext (out, ringtoneAvailSmpl, spkrVolume);
+            write (out, ringtoneAvailBytes, _RingtoneHandle);
+            free (out);
+        }
+
+        out = NULL;
+
+    } else if (_RingtoneHandle) {
+
+        int ringtoneAvailSmpl = snd_pcm_avail_update (_RingtoneHandle);
+        int ringtoneAvailBytes = ringtoneAvailSmpl*sizeof (SFLDataFormat);
+
+        out = (SFLDataFormat *) malloc (ringtoneAvailBytes);
+
+        if (out) {
+            memset (out, 0, ringtoneAvailBytes);
+            write (out, ringtoneAvailBytes, _RingtoneHandle);
+            free (out);
+        }
+
+        out = NULL;
+    }
+
     // Additionally handle the mic's audio stream
     int micAvailBytes;
-
     int micAvailPut;
-
     int toPut;
 
-    SFLDataFormat* in;
-
-
-    // snd_pcm_sframes_t micAvailAlsa;
-    in = 0;
+    SFLDataFormat* in = NULL;
 
     if (is_capture_running()) {
 
         micAvailBytes = snd_pcm_avail_update (_CaptureHandle);
-        // _debug("micAvailBytes %i", micAvailBytes);
+        // _debug("CAPTURE: %i", micAvailBytes);
 
         if (micAvailBytes > 0) {
             micAvailPut = getMainBuffer()->availForPut();
@@ -990,9 +1087,9 @@ void AlsaLayer::audioCallback (void)
             in = (SFLDataFormat*) malloc (toPut * sizeof (SFLDataFormat));
             toPut = read (in, toPut* sizeof (SFLDataFormat));
 
-            adjustVolume (in, toPut, SFL_PCM_CAPTURE);
+            if (in) {
+                adjustVolume (in, toPut, SFL_PCM_CAPTURE);
 
-            if (in != 0) {
                 int _mainBufferSampleRate = getMainBuffer()->getInternalSamplingRate();
 
                 if (_mainBufferSampleRate && ( (int) _audioSampleRate != _mainBufferSampleRate)) {
@@ -1005,7 +1102,7 @@ void AlsaLayer::audioCallback (void)
                     // _debug("nb_sample_up %i", nb_sample_up);
                     nbSample = _converter->downsampleData ( (SFLDataFormat*) in, rsmpl_out, _mainBufferSampleRate, _audioSampleRate, nb_sample_up);
 
-                    dcblocker->filter_signal (rsmpl_out, nbSample);
+                    _audiofilter->processAudio (rsmpl_out, nbSample*sizeof (SFLDataFormat));
 
                     getMainBuffer()->putData (rsmpl_out, nbSample * sizeof (SFLDataFormat), 100);
 
@@ -1014,7 +1111,16 @@ void AlsaLayer::audioCallback (void)
 
                 } else {
 
-                    getMainBuffer()->putData (in, toPut, 100);
+
+                    SFLDataFormat* filter_out = (SFLDataFormat*) malloc (framesPerBufferAlsa * sizeof (SFLDataFormat));
+
+                    if (filter_out) {
+                        _audiofilter->processAudio (in, filter_out, toPut);
+                        // captureFile->write ( (const char *) filter_out, toPut);
+                        getMainBuffer()->putData (filter_out, toPut, 100);
+                        free (filter_out);
+                    }
+
                 }
             }
 
@@ -1022,7 +1128,7 @@ void AlsaLayer::audioCallback (void)
 
             in=0;
         } else if (micAvailBytes < 0) {
-            _debug ("AlsaLayer::audioCallback (mic): error: %s", snd_strerror (micAvailBytes));
+            _debug ("Audio: Mic error: %s", snd_strerror (micAvailBytes));
         }
 
     }
